syntax = "proto3";

package import_cousin_package_same_name.test.subpackage;

import "cousin.proto";

// Verify that we can import a message unrelated to us, in a subpackage with the same name as us.

message Test {
    cousin.subpackage.CousinMessage message = 1;
}
