syntax = "proto3";

package deprecated;

// Some documentation about the Test message.
message Test {
    Message message = 1 [deprecated=true];
    int32 value = 2;
}

message Message {
    option deprecated = true;
    string value = 1;
}
