syntax = "proto3";

package repeated_duration_timestamp;

import "google/protobuf/duration.proto";
import "google/protobuf/timestamp.proto";


message Test {
    repeated google.protobuf.Timestamp times = 1;
    repeated google.protobuf.Duration durations = 2;
}
